// Gaurav Jain
// SE 382 - Assignment 1
// Goon.cpp

#include "Goon.h"

#include <iostream>

#define blockWidth 40
#define blockHeight 30

// Constructor
Goon::Goon(int x, int y, Level *lev, XInfo &xinfo) : Person(x, y, lev, xinfo) {
	freeFall = false;
	firstPaint = true;
}

// Destructor
Goon::~Goon() {

	// Free up Pixmap
	XFreePixmap(xinfo->display, leftPos1);
	XFreePixmap(xinfo->display, leftPos2);
	XFreePixmap(xinfo->display, leftPos3);
	XFreePixmap(xinfo->display, rightPos1);
	XFreePixmap(xinfo->display, rightPos2);
	XFreePixmap(xinfo->display, rightPos3);
	XFreePixmap(xinfo->display, upPos1);
	XFreePixmap(xinfo->display, upPos2);
	XFreePixmap(xinfo->display, downPos1);
	XFreePixmap(xinfo->display, downPos2);
	XFreePixmap(xinfo->display, bar1);
}

// Paint the goon. Based on direction/location of goon call a different paint method that will in turn
// call a different bitmap file to load
void Goon::paint(XInfo &xinfo) {
	if(firstPaint) {
		readBitMaps(xinfo);
		firstPaint = false;
		paint(xinfo);
	} 
	else {
		if(level->getBlockType(getLoc().xCoord, getLoc().yCoord) == 'B') {
			paintBar(xinfo);
		}
		else if(moveAction == LEFT) {
			paintLeft(xinfo);
		}
		else if(moveAction == RIGHT) {
			paintRight(xinfo);
		}
		else if(moveAction == UP) {
			paintUp(xinfo);
		}
		else if(moveAction == DOWN) {
			paintDown(xinfo);
		} 
		
		leftX=posX; leftY=posY; rightX=posX+29; rightY=posY+29;
	}
}

void Goon::readBitMaps(XInfo &xinfo) {
	// Read all Image files that I need to animate runner
	unsigned int pic_width = 0, pic_height = 0;
	int x_hot_return, y_hot_return;
	
	// Read bitmaps for moving left
	XReadBitmapFile(xinfo.display, xinfo.window, "./images/goon_left.xbm", &pic_width, &pic_height, &leftPos1, &x_hot_return, &y_hot_return);
	XReadBitmapFile(xinfo.display, xinfo.window, "./images/goon_left.xbm", &pic_width, &pic_height, &leftPos2, &x_hot_return, &y_hot_return);
	XReadBitmapFile(xinfo.display, xinfo.window, "./images/goon_left.xbm", &pic_width, &pic_height, &leftPos3, &x_hot_return, &y_hot_return);
	
	// Read bitmaps for moving left
	XReadBitmapFile(xinfo.display, xinfo.window, "./images/goon_left_mask.xbm", &pic_width, &pic_height, &leftPos1Mask, &x_hot_return, &y_hot_return);
	XReadBitmapFile(xinfo.display, xinfo.window, "./images/goon_left_mask.xbm", &pic_width, &pic_height, &leftPos2Mask, &x_hot_return, &y_hot_return);
	XReadBitmapFile(xinfo.display, xinfo.window, "./images/goon_left_mask.xbm", &pic_width, &pic_height, &leftPos3Mask, &x_hot_return, &y_hot_return);
	
	
	// Read bitmaps for moving right
	XReadBitmapFile(xinfo.display, xinfo.window, "./images/goon_right.xbm", &pic_width, &pic_height, &rightPos1, &x_hot_return, &y_hot_return);
	XReadBitmapFile(xinfo.display, xinfo.window, "./images/goon_right.xbm", &pic_width, &pic_height, &rightPos2, &x_hot_return, &y_hot_return);
	XReadBitmapFile(xinfo.display, xinfo.window, "./images/goon_right.xbm", &pic_width, &pic_height, &rightPos3, &x_hot_return, &y_hot_return);
		
		
	// Read bitmaps for moving right
	XReadBitmapFile(xinfo.display, xinfo.window, "./images/goon_right_mask.xbm", &pic_width, &pic_height, &rightPos1Mask, &x_hot_return, &y_hot_return);
	XReadBitmapFile(xinfo.display, xinfo.window, "./images/goon_right_mask.xbm", &pic_width, &pic_height, &rightPos2Mask, &x_hot_return, &y_hot_return);
	XReadBitmapFile(xinfo.display, xinfo.window, "./images/goon_right_mask.xbm", &pic_width, &pic_height, &rightPos3Mask, &x_hot_return, &y_hot_return);
	
	
	// Read bitmaps for going up - we have to use duplicate bitmaps for consistency purposes - all Pixmaps need to be
	// defined or we could have a segfault
	XReadBitmapFile(xinfo.display, xinfo.window, "./images/goon_up.xbm", &pic_width, &pic_height, &upPos1, &x_hot_return, &y_hot_return);
	XReadBitmapFile(xinfo.display, xinfo.window, "./images/goon_up.xbm", &pic_width, &pic_height, &upPos2, &x_hot_return, &y_hot_return);
	
	// Read bitmaps for going up - we have to use duplicate bitmaps for consistency purposes - all Pixmaps need to be
	// defined or we could have a segfault
	XReadBitmapFile(xinfo.display, xinfo.window, "./images/goon_up_mask.xbm", &pic_width, &pic_height, &upPos1Mask, &x_hot_return, &y_hot_return);
	XReadBitmapFile(xinfo.display, xinfo.window, "./images/goon_up_mask.xbm", &pic_width, &pic_height, &upPos2Mask, &x_hot_return, &y_hot_return);
		
	// Read bitmaps for going down
	XReadBitmapFile(xinfo.display, xinfo.window, "./images/goon_up.xbm", &pic_width, &pic_height, &downPos1, &x_hot_return, &y_hot_return);
	XReadBitmapFile(xinfo.display, xinfo.window, "./images/goon_up.xbm", &pic_width, &pic_height, &downPos2, &x_hot_return, &y_hot_return);
	
	// Read bitmaps for going down
	XReadBitmapFile(xinfo.display, xinfo.window, "./images/goon_up_mask.xbm", &pic_width, &pic_height, &downPos1Mask, &x_hot_return, &y_hot_return);
	XReadBitmapFile(xinfo.display, xinfo.window, "./images/goon_up_mask.xbm", &pic_width, &pic_height, &downPos2Mask, &x_hot_return, &y_hot_return);
	
	// Read bitmap for when on bar
	XReadBitmapFile(xinfo.display, xinfo.window, "./images/goon_bar.xbm", &pic_width, &pic_height, &bar1, &x_hot_return, &y_hot_return);

	// Read bitmap for when on bar
	XReadBitmapFile(xinfo.display, xinfo.window, "./images/goon_bar_mask.xbm", &pic_width, &pic_height, &bar1Mask, &x_hot_return, &y_hot_return);
}	
	
// Goon needs to drop gold, check if it can be
int Goon::dropGold() {
	int goldBlock = -1;
	int myBlock = level->mapPointsToBlock(posX, posY);
	// Determine if I have gold and I am standing on ground
	if(((level->getBlockType(myBlock) == 'S') && (level->getBlockType(myBlock + 800/blockWidth) == 'F')) && (gold>0)) {
		// Always drop gold behind you
		if(moveAction == LEFT) {
			goldBlock = myBlock + 1;
		}
		else {
			goldBlock = myBlock - 1;
		}
		
		if(level->getBlockType(goldBlock) == 'S') {
			gold--;
			level->setBlockType(goldBlock, 'G');
			return (goldBlock);
		}
	}	
	return -1;
}