// Gaurav Jain
// SE 382 - Assignment 1
// Level.cpp

// Level class that keeps track of all components within it and the location of runner and goons

#include "Level.h"
#include "Runner.h"
#include "Goon.h"

#include <fstream>
#include <iostream>
#include <sstream>

#define blockWidth 40
#define blockHeight 30


// Constructor for Level class

Level::Level(string fileName, XInfo &xinfo, int numRunnerLives) {
	
	firstGoldPaint = true;
	myXInfo = &xinfo;
	
	// Used for file input
	ifstream infile;
	string input;
	string gridFile;
	infile.open(fileName.c_str());
	
	infile >> gridFile;
	infile >> input;
	
	// Set runner settings
	// Set initial X
	initialXRunner = atoi(input.c_str());
	
	// Set initial Y
	infile >> input;
	initialYRunner = atoi(input.c_str());
	
	// Find out how many goons we have in the level
	infile >> input;
	int numGoons = atoi(input.c_str());
	
	// For each goon in level, read the starting location and add to vector
	for(int i=1; i <= numGoons ; i++) {
		infile >> input;
		xCoordsGoons.push_back(atoi(input.c_str()));
		infile >> input;
		yCoordsGoons.push_back(atoi(input.c_str()));
	}
	
	infile.close();
	readMapFile(gridFile); // first line in the Level file is the name of file containing grid
	
	// Create runners and goons
	createRunner(initialXRunner, initialYRunner, xinfo);
	createGoons(xCoordsGoons, yCoordsGoons, xinfo);
	
	// initialize private members
	totalGoldCount = countGoldInLevel();
	goonGoldDropCount = 1;
	
	// How many lives does the runner have remaining?
	runnerLives = numRunnerLives;
	
	
	// Paint entire level
	paint(xinfo);
}


/* -=-=-=- Destructor -=-=-=--=-=-=- */
Level::~Level() {

	delete runner; // Delete runner object
	for(vector<Goon*>::iterator iter = goons.begin(); iter != goons.end(); iter++) {
		delete *iter; // Delete all goons in level
	}
}

/* -=-=-=- End of destructor -=-=-=-=-=-=-=-=- */

/* -=-=-=-  Read the file that contains grid information =-=-=-= */
void Level::readMapFile(string fileName) {
	ifstream infile;
	string input;
	infile.open(fileName.c_str());
	while(true) {
		getline(infile, input);
		if(infile.fail()) {
			infile.close();
			break;
		}
		levelMap.push_back(input); // Add block information to vector
	}
}

/* -=-=-=- End: readMapFile -=-=-=-=-=-=-=--=- */

/* =--=-=-=-=- Create Runner and Goons -=-=-=-=--=-=-=-=-=-=-=-- */
void Level::createRunner(int posX, int posY, XInfo &xinfo) {
	runner = new Runner(posX, posY, this, xinfo);
	//runner->paint(xinfo);
}

void Level::createGoons(vector<int> xCoords, vector<int> yCoords, XInfo &xinfo) {
	vector<int>::iterator ycoordIter = yCoords.begin();
	for(vector<int>::iterator xcoordIter = xCoords.begin(); xcoordIter != xCoords.end(); xcoordIter++) {
		goons.push_back(new Goon(*xcoordIter, *ycoordIter, this, xinfo));
		ycoordIter++;
	}
}
/* -=-=-=-=-=-= End Creating Runner and Goons -=-=-=-=-=-=-=-=-=-=- */


/* =-=-=-=-=--=- This function finds out how many gold bags we have in this level */
int Level::countGoldInLevel() {
	int count = 0;
	for(vector<string>::iterator levelMapIter = levelMap.begin(); levelMapIter != levelMap.end(); levelMapIter++) {
		string curRow = *levelMapIter;
		for(int i=0; i<curRow.length();i++) {
			if(curRow[i] == 'G') { // Is current block a gold bag?
				count++;
			}
		}
	}
	return count;
}
/* =-=-=-=-=-= End: countGoldInLevel() -=-=-=-=-=--=- */

/* -=-=--=- Paint Level -=-=-=-=--=-=-=- */

void Level::paint(XInfo &xinfo) {
		XClearWindow(xinfo.display, xinfo.window);
		drawRunner(xinfo); // Default direction is left
		drawGoons(xinfo);
		drawMap(xinfo);
		displayStatusInformation(xinfo); // display number of gold bags collected so far
		displayStaticMessages(xinfo);
}

/* -=-=-=-=- End paint level -=-=-=-=-=-=-= */

/* =-=-=-=-=-=- Draw Runner and Goons --=-=-=-=-=-= */

void Level::drawRunner(XInfo &xinfo) {
	runner->paint(xinfo);
}

void Level::drawGoons(XInfo &xinfo) {
	for(vector<Goon*>::iterator goonIter = goons.begin(); goonIter != goons.end(); goonIter++) {
		//drawGoon(xinfo, *goonIter);
		(*goonIter)->paint(xinfo);
		dropGold(xinfo, *goonIter); // See if this goon would like to drop a gold bag
	}
}

/* -=-=-=-=-=-=- End Drawing Runner and Goons -=-=-=-=-=- */


/* -=-=-=- Goon drops gold =-=--=-=-=- */

void Level::dropGold(XInfo &xinfo, Goon* goon) {
	// Determine if it is time to drop a gold bag
	if(goonGoldDropCount % 500 == 0) {
			int goldDropBlock = goon->dropGold();
			if(goldDropBlock != -1)
				drawGold(xinfo, goldDropBlock);
	}
	goonGoldDropCount++;
}
/* -=-=-= End:dropGold -=-=-=-=- */


/* -=-=-  Draw the level based on grid information read -=-=-=- */

void Level::drawMap(XInfo &xinfo) {
	int blockCount=0;
	for(vector<string>::iterator levelMapIter = levelMap.begin(); levelMapIter != levelMap.end(); levelMapIter++) {
		string curRow = *levelMapIter;
		for(int i=0; i<curRow.length();i++) {
			if(curRow[i] == 'S') {
				// Space - so leave it blank
				blockCount++;
			}
			else if(curRow[i] == 'F') {
				drawFloor(xinfo, blockCount);
				blockCount++;
			}
			else if(curRow[i] == 'B') {
			
				drawBar(xinfo, blockCount);
				blockCount++;
			}
			else if(curRow[i] == 'L') {
				drawLadder(xinfo, blockCount);
				blockCount++;
			}
			else if(curRow[i] == 'G') {
				drawGold(xinfo, blockCount);
				blockCount++;
			}
		}
	}
}

/* -=-=- End: drawMap() -=-==-=-=- */


/* -=-=-=- Information to be displayed on screen -=-=-=-=- */

// Display information on the bottom of the screen
void Level::displayStatusInformation(XInfo &xinfo) {
	
	int goldStartX = 195;
	int goldStartY = 535;
	stringstream ss, ss2, ss3;
	ss << runner->getNumGold();
	ss2 << totalGoldCount - runner->getNumGold();
	string goldString = ss.str();
	string goldLeftString = ss2.str();
	
	// Text messages to be displayed
	Displayable *goldMsg = new Text(10, 535, "Number of Gold Bags Collected: ");
	Displayable *goldLeftMsg = new Text(10, 560, "Number of Gold Bags Left: ");
	
	// Values updated regularly
	Displayable *goldCount = new Text(goldStartX, goldStartY, goldString);
	Displayable *goldLeftCount = new Text(goldStartX - 25, goldStartY + 25, goldLeftString);
	
	ss3 << runnerLives;
	string numLivesString = ss3.str();
	Displayable *runnerLives = new Text(10, 585, "Number of runner lives remaining: ");
	Displayable *numRunnerLives = new Text(goldStartX + 15, 585, numLivesString);
	
	// Repaint 
	repaintBlock(xinfo, mapPointsToBlock(goldStartX, goldStartY));
	repaintBlock(xinfo, mapPointsToBlock(goldStartX - 25, goldStartY + 25));
	repaintBlock(xinfo, mapPointsToBlock(goldStartX + 15, 585));
	
	// Paint number of gold bags picked up
	goldMsg->paint(xinfo);
	goldCount->paint(xinfo);
	
	// Paint number of gold bags left
	goldLeftMsg->paint(xinfo);
	goldLeftCount->paint(xinfo);
	
	// Paint number of runner lives remaining
	runnerLives->paint(xinfo);
	numRunnerLives->paint(xinfo);
	

}

void Level::displayStaticMessages(XInfo &xinfo) {
	Displayable *msg1 = new Text(600, 550, "g - Forcibly kill random goon");
	Displayable *msg2 = new Text(600, 565, "a - Advance to next level");
	Displayable *msg3 = new Text(600, 580, "c - Increase runner's life by 1");
	msg1->paint(xinfo);
	msg2->paint(xinfo);
	msg3->paint(xinfo);
}

void Level::showAdvanceLevelMsg(XInfo &xinfo) {
	XClearWindow(xinfo.display, xinfo.window);
	Displayable *msg = new Text(290, 275, "Advancing to next level...");
	msg->paint(xinfo);
	XFlush(xinfo.display);
	sleep(3);
}
	
/* -=-=-- End of display information on screen -=-=-=-=-=-=-*/

/* -=-= Draw Level Components: Floor, Bar and Ladder, Gold and display Status information at bottom -=-=-=-=- */

// Use X functions to draw various figures
void Level::drawFloor(XInfo &xinfo, int block) {
	XFillRectangle(xinfo.display, xinfo.window, xinfo.gc, mapBlockToX(block), mapBlockToY(block), blockWidth, blockHeight);
}

void Level::drawBar(XInfo &xinfo, int block) {
	XDrawLine(xinfo.display, xinfo.window, xinfo.gc, mapBlockToX(block), mapBlockToY(block) + (blockHeight/5), mapBlockToX(block+1) - 1, mapBlockToY(block) + (blockHeight/5));
}

void Level::drawLadder(XInfo &xinfo, int block) {
	int startX = mapBlockToX(block);
	int startY = mapBlockToY(block);
	XDrawRectangle(xinfo.display, xinfo.window, xinfo.gc, startX, startY, (blockWidth/10) - 1, blockHeight - 1); // Draw left vertical bar
	XDrawRectangle(xinfo.display, xinfo.window, xinfo.gc, (startX + blockWidth) - (blockWidth/10), startY, (blockWidth/10) - 1, blockHeight - 1); // Draw right vertical bar
	XDrawRectangle(xinfo.display, xinfo.window, xinfo.gc, startX + (blockWidth/10) - 1, startY + (blockHeight/4), blockWidth - ((blockWidth/10)*2) + 1, blockHeight/10); // Draw upper horizontal bar
	XDrawRectangle(xinfo.display, xinfo.window, xinfo.gc, startX + (blockWidth/10) - 1 , startY + ((blockHeight/4)*3), blockWidth - ((blockWidth/10)*2) + 1, blockHeight/10); // Draw lower horizontal bar

}

void Level::drawGold(XInfo &xinfo, int block) {
	int startX = mapBlockToX(block);
	int startY = mapBlockToY(block);	
	if(firstGoldPaint) {
		readGoldBitmap(xinfo);
		firstGoldPaint = false;
		drawGold(xinfo, block);
	}
	else {
		XCopyPlane(xinfo.display, goldImage, xinfo.window, xinfo.gc, 0, 0, 35, 30, startX, startY, 1);
	}
}

void Level::readGoldBitmap(XInfo &xinfo) {
	unsigned int pic_width = 0, pic_height = 0;
	int x_hot_return, y_hot_return;
	
	// Read bitmaps for gold bag
	XReadBitmapFile(xinfo.display, xinfo.window, "./images/goldbag.xbm", &pic_width, &pic_height, &goldImage, &x_hot_return, &y_hot_return);
	
}


	
/* =-=-=-=-=-=- End Drawing Components -=-=-=-=-=-=-==-=-=-=-=-=-=-=-=-=- */


/* =-=-==--=-=- Move Runner and Goons -=-=-=-=-=-=-=- */

void Level::moveRunner(XInfo &xinfo, Action moveAction) {
	movePoints points;
	if(runner->isFreeFalling()) {
		points = runner->move(DOWN);
	}
	else {
		points = runner->move(moveAction);
	}
	
	// If the move is legal, only then repaint
	if(points.start.leftX > 0) {
		int block1 = mapPointsToBlock(points.start.leftX, points.start.leftY);
		int block2 = mapPointsToBlock(points.start.rightX, points.start.leftY);
		int block3 = mapPointsToBlock(points.start.leftX, points.start.rightY);
		int block4 = mapPointsToBlock(points.start.rightX, points.start.rightY);
			
		// Clear all 4 blocks around the runner
		XClearArea(xinfo.display, xinfo.window, mapBlockToX(block1), mapBlockToY(block1), blockWidth, blockHeight, false);
		XClearArea(xinfo.display, xinfo.window, mapBlockToX(block2), mapBlockToY(block2), blockWidth, blockHeight, false);
		XClearArea(xinfo.display, xinfo.window, mapBlockToX(block3), mapBlockToY(block3), blockWidth, blockHeight, false);
		XClearArea(xinfo.display, xinfo.window, mapBlockToX(block4), mapBlockToY(block4), blockWidth, blockHeight, false);
		
		// Draw all the cleared blocks around runner
		drawComponent(xinfo, getComponent(block1), block1);
		drawComponent(xinfo, getComponent(block2), block2);
		drawComponent(xinfo, getComponent(block3), block3);
		drawComponent(xinfo, getComponent(block4), block4);
		
		runner->paint(xinfo);
	}
	
	// Redisplay status information
	displayStatusInformation(xinfo);
}

// Move the runner forward
void Level::moveRunner(Action moveAction) {
	moveRunner(*myXInfo, moveAction);
}

// Move the goons forward
void Level::moveGoons(XInfo &xinfo) {
	movePoints points;
	vector<int> blocks;
	// Get runners location
	Location runLoc = runner->getLoc();
	// Get goons location and make appropriate move decision
		
	for(vector<Goon*>::iterator goonIter = goons.begin(); goonIter != goons.end(); goonIter++) {
		Location goonLoc = (*goonIter)->getLoc();
		Action moveAction = makeDecision(goonLoc, runLoc, *goonIter);
		if((*goonIter)->isFreeFalling()) {
			points = (*goonIter)->move(DOWN);
		}
		else {
			points = (*goonIter)->move(moveAction);
		}
		
		// If the move is legal, only then repaint
		if(points.start.leftX > 0) {
			// Blocks to be repainted around goon	
			int block1 = mapPointsToBlock(points.start.leftX, points.start.leftY);
			int block2 = mapPointsToBlock(points.start.rightX, points.start.leftY);
			int block3 = mapPointsToBlock(points.start.leftX, points.start.rightY);
			int block4 = mapPointsToBlock(points.start.rightX, points.start.rightY);
				
			/*// Blocks to be repainted around runner
			int block5 = mapPointsToBlock(runLoc.xCoord, runLoc.yCoord);
			int block6 = mapPointsToBlock(runLoc.xCoord + 39, runLoc.yCoord);
			int block7 = mapPointsToBlock(runLoc.xCoord, runLoc.yCoord + 29);
			int block8 = mapPointsToBlock(runLoc.xCoord + 39, runLoc.yCoord + 29);*/
			
			// Clear blocks around goon
			XClearArea(xinfo.display, xinfo.window, mapBlockToX(block1), mapBlockToY(block1), blockWidth, blockHeight, false);
			XClearArea(xinfo.display, xinfo.window, mapBlockToX(block2), mapBlockToY(block2), blockWidth, blockHeight, false);
			XClearArea(xinfo.display, xinfo.window, mapBlockToX(block3), mapBlockToY(block3), blockWidth, blockHeight, false);
			XClearArea(xinfo.display, xinfo.window, mapBlockToX(block4), mapBlockToY(block4), blockWidth, blockHeight, false);
		
		/*// Clear blocks around runner
		XClearArea(xinfo.display, xinfo.window, mapBlockToX(block5), mapBlockToY(block5), blockWidth, blockHeight, false);
			XClearArea(xinfo.display, xinfo.window, mapBlockToX(block6), mapBlockToY(block6), blockWidth, blockHeight, false);
			XClearArea(xinfo.display, xinfo.window, mapBlockToX(block7), mapBlockToY(block7), blockWidth, blockHeight, false);
			XClearArea(xinfo.display, xinfo.window, mapBlockToX(block8), mapBlockToY(block8), blockWidth, blockHeight, false);*/
		
			// Draw all blocks cleared above
			drawComponent(xinfo, getComponent(block1), block1);
			drawComponent(xinfo, getComponent(block2), block2);
			drawComponent(xinfo, getComponent(block3), block3);
			drawComponent(xinfo, getComponent(block4), block4);
			/*drawComponent(xinfo, getComponent(block5), block5);
			drawComponent(xinfo, getComponent(block6), block6);
			drawComponent(xinfo, getComponent(block7), block7);
			drawComponent(xinfo, getComponent(block8), block8);*/
		}
	
		
	}
	
	// We have repainted all blocks that were affected by the move. Now draw the player and goons back.
	drawGoons(xinfo);
	//runner->paint(xinfo);
}

// Determine what direction the goon should travel in
Action Level::makeDecision(Location locStart, Location locEnd, Person* person) {
	if((locEnd.yCoord > locStart.yCoord) && (person->verifyMove(DOWN))) {
		return DOWN;
	}
	else if((locEnd.yCoord < locStart.yCoord) && (person->verifyMove(UP))) {
		return UP;
	}
	else if((locEnd.xCoord > locStart.xCoord) && (person->verifyMove(RIGHT))) {
		return RIGHT;
	}
	else if((locEnd.xCoord < locStart.xCoord) && (person->verifyMove(LEFT))) {
		return LEFT;
	}
	
}


/* -=-=--=-=-=- End move runner and goons -=-=-=-=-=-=-=-=-=- */

/* -=-=-- Helper functions for doing some math calculations -=-=-=--=-= */

int Level::mapPointsToBlock(int x, int y) {
	int block;
	block = ((y/blockHeight)*(800/blockWidth)) + (x/blockWidth);
	return block;
}

int Level::mapBlockToX(int block) {
	int x;
	x = (block%(800/blockWidth))*blockWidth;
	return x;
}

int Level::mapBlockToY(int block) {
	int y;
	y = (block/(800/blockWidth))*blockHeight;
	return y;
}

char Level::getComponent(int block) {
	string row = levelMap[block/(800/blockWidth)];
	return (row[block%(800/blockWidth)]);
}

void Level::setComponent(int block, char blockType) {
	if(block>=0) {
		string row = levelMap[block/(800/blockWidth)];
		row[block%(800/blockWidth)] = blockType;
		levelMap[block/(800/blockWidth)] = row;
	}
}

/* -=-=-=-=- End Math Helper functions -=-=-=-=-=-=-=-==-=-=-=-=-=-=- */


/* =-=-=-=-=- Repaint Methods -=-=-=-=-=- */

/* These methods are used to move repaint a section of the canvas that is modified after a runner/goon is moved */


void Level::repaintBlock(XInfo &xinfo, int block) {
	XClearArea(xinfo.display, xinfo.window, mapBlockToX(block), mapBlockToY(block), blockWidth, blockHeight, false);
	drawComponent(xinfo, getComponent(block), block);
}

char Level::getBlockType(int x, int y) {
	int block = mapPointsToBlock(x, y);
	return getBlockType(block);
}

char Level::getBlockType(int block) {
	return getComponent(block);
}

void Level::setBlockType(int x, int y, char blockType) {
	int block = mapPointsToBlock(x, y);
	setBlockType(block, blockType);
}

void Level::setBlockType(int block, char blockType) {
	if(block >= 0) {
		setComponent(block, blockType);
	}
}

void Level::drawComponent(XInfo &xinfo, char comp, int block) {
	if(comp == 'S') {
		// Space - leave blank
	}
	else if(comp == 'F') {
		drawFloor(xinfo, block);
	}
	else if(comp == 'B') {
		drawBar(xinfo, block);
	}
	else if(comp == 'L') {
		drawLadder(xinfo, block);
	}
	else if(comp == 'G') {
		drawGold(xinfo, block);
	}
}

/* -=-=--=-=-= End Repaint Methods -=-=-=-=--= */

// Find out if runner and goon have collided
bool Level::checkCollision() {
	Extremeties runnerLoc = runner->getExts();
	for(vector<Goon*>::iterator goonIter = goons.begin(); goonIter != goons.end(); goonIter++) {
		Extremeties goonLoc = (*goonIter)->getExts();
		
		// Goon's upper edge is in between the runner
		if((goonLoc.leftY >= runnerLoc.leftY) && (goonLoc.leftY <= runnerLoc.rightY)) {
			// If Goon's side edges is in between the runner as well?
			if((goonLoc.rightX >= runnerLoc.leftX) && (goonLoc.rightX <= runnerLoc.rightX)) {
				return true;
			}
			else if((goonLoc.leftX >= runnerLoc.leftX) && (goonLoc.leftX <= runnerLoc.rightX)) {
				return true;
			}
		}
		// Goon's lower edge is in between the runner
		else if((goonLoc.rightY >= runnerLoc.leftY) && (goonLoc.rightY <= runnerLoc.rightY)) {
			// Check if goon's side edges are in between as well
			if((goonLoc.leftX >= runnerLoc.leftX) && (goonLoc.leftX <= runnerLoc.rightX)) {
				return true;
			}
			else if((goonLoc.rightX >= runnerLoc.leftX) && (goonLoc.rightX <= runnerLoc.rightX)) {
				return true;
			}
		}
	}
	return false;
}

// Find out if a person is present at the given location - preventing collision amongst goons
bool Level::noPerson(int x, int y) {
	for(vector<Goon*>::iterator goonIter = goons.begin(); goonIter != goons.end(); goonIter++) {
		Extremeties goonExts = (*goonIter)->getExts();
		int leftX = mapBlockToX(mapPointsToBlock(goonExts.leftX, goonExts.leftY));
		int leftY = mapBlockToY(mapPointsToBlock(goonExts.leftX, goonExts.leftY));
		int rightX = mapBlockToX(mapPointsToBlock(goonExts.rightX, goonExts.rightY));
		int rightY = mapBlockToY(mapPointsToBlock(goonExts.rightX, goonExts.rightY));
		
		if(((x>= leftX) && (x<= rightX)) && ((y >= leftY) && (y <= rightY))) {
			return false;
		}
	}
	return true;
}

/* -=-=- Find out if a runner is at a certain location or not -=-=-= */
bool Level::noPersonAtLoc(int x, int y) {
	int leftX, leftY, rightX, rightY;
	Extremeties runExts = runner->getExts();
	leftX = mapBlockToX(mapPointsToBlock(runExts.leftX, runExts.leftY));
	leftY = mapBlockToY(mapPointsToBlock(runExts.leftX, runExts.leftY));
	rightX = mapBlockToX(mapPointsToBlock(runExts.rightX, runExts.rightY) + 1);
	rightY = mapBlockToY(mapPointsToBlock(runExts.rightX, runExts.rightY) + (800/blockWidth));
	
	if(((x>= leftX) && (x<= rightX)) && ((y >= leftY) && (y <= rightY))) {
		return false;
	}
	
	for(vector<Goon*>::iterator goonIter = goons.begin(); goonIter != goons.end(); goonIter++) {
		Extremeties goonExts = (*goonIter)->getExts();
		leftX = mapBlockToX(mapPointsToBlock(goonExts.leftX, goonExts.leftY));
		leftY = mapBlockToY(mapPointsToBlock(goonExts.leftX, goonExts.leftY));
		rightX = mapBlockToX(mapPointsToBlock(goonExts.rightX, goonExts.rightY) + 1);
		rightY = mapBlockToY(mapPointsToBlock(goonExts.rightX, goonExts.rightY) + (800/blockWidth));
		
		if(((x>= leftX) && (x<= rightX)) && ((y >= leftY) && (y <= rightY))) {
			return false;
		}
	}
	return true;
}
/* -=-=-=- End: noRunnerAtLoc() -=-=-=-=- */
	
/* This function is purely designed to make levels easier as and when the user wishes. */

void Level::getRidOfRandomGoon(XInfo &xinfo) {
	if(!goons.empty()) {
		
		// Find out what goon we want to delete
		Goon* delGoon = goons.back();
		Extremeties points = delGoon->getExts();
		
		// Find the blocks the goon is using
		int block1 = mapPointsToBlock(points.leftX, points.leftY);
		int block2 = mapPointsToBlock(points.rightX, points.leftY);
		int block3 = mapPointsToBlock(points.leftX, points.rightY);
		int block4 = mapPointsToBlock(points.rightX, points.rightY);
		
		// repaint each block that the goon is using
		repaintBlock(xinfo, block1);
		repaintBlock(xinfo, block2);
		repaintBlock(xinfo, block3);
		repaintBlock(xinfo, block4);
		
		// remove the goon from level
		goons.pop_back();
	}
}	

// Set the number of lives runner has available. Used in changing the number of lives available
void Level::setRunnerLives(XInfo &xinfo, int num) {
	runnerLives = num; // required if you are going to increase the number of lives before a level is deleted
	displayStatusInformation(xinfo);
}

// Runner needs help in determining what direction he/she should go in. Take the coordinates and tell 
// the runner what direction to head in.
Action Level::makeRunnerMoveDecision(int x, int y) {
	Location curLoc = runner->getLoc();
	Location destLoc;
	destLoc.xCoord = x;
	destLoc.yCoord = y;	
	Action decision = makeDecision(curLoc, destLoc, runner);
	return decision;
}
	
	
// Set the number of pixels to move for each person
void Level::setMovePixel(int movePixel) {
	this->movePixel = movePixel;
}

// Get the number of pixels each person should move
int Level::getMovePixel() {
	return movePixel;
}

/* -=-=- This function returns true if runner has picked up all the gold bags. False otherwise =-=- */

bool Level::runnerHasAllGold() {
	return(totalGoldCount == runner->getNumGold());
}

/* =--=-- End: runnerHasAllGold() -=-=-=-- */