// Gaurav Jain
// SE 382 - Assignment 1
// Level.h

#ifndef _LEVEL_H_
#define _LEVEL_H_

#include "Displayable.h"
#include "global_enum_defs.h"
#include "Text.h"

#include <string>
#include <vector>

using namespace std;

class Person;
class Runner;
class Goon;

// Specify the Location of runner or goon - boiled down version of Extremeties
struct Location {
	int xCoord;
	int yCoord;
};

// Specify the four extremeties of runner or goon
struct Extremeties {
	int leftX;
	int leftY;
	int rightX;
	int rightY;
}; 

class Level: public Displayable {
	public:
		Level(string fileName, XInfo &xinfo, int numRunnerLives);
		~Level();
		void paint(XInfo &xinfo);
		void readMapFile(string fileName);
		void drawMap(XInfo &xinfo);
		
		// Move runner and goons
		void moveRunner(XInfo &xinfo, Action moveAction);
		void moveRunner(Action moveAction);
		void moveGoons(XInfo &xinfo);
		
		int mapPointsToBlock(int x, int y);
		
		char getBlockType(int x, int y); // get a block type for a certain location on the map
		char getBlockType(int block); // overloaded version of above
		
		void setBlockType(int x, int y, char blockType); // changes a block in my grid
		void setBlockType(int block, char blockType); // overloaded function of above
		
		bool checkCollision(); // checks for collisions between goon and runner
		bool noPerson(int x, int y); // tells if a goon is present at the given location
		bool noPersonAtLoc(int x, int y); // tells if a runner is present at the given location
		
		void getRidOfRandomGoon(XInfo &xinfo); // deletes random goon from level to make it slightly easier
		
		void displayStatusInformation(XInfo &xinfo); // Display status information at bottom of screen
		
		void setRunnerLives(XInfo &xinfo, int num); //change the number of runner lives while playing the game
		
		Action makeRunnerMoveDecision(int x, int y); // decide which way runner should run
		
		void setMovePixel(int movePixel);  // sets the number of pixels to move for each person
		int getMovePixel(); // returns the number of pixels each person should move
		
		// Math Helper functions
		
		int mapBlockToX(int block);
		int mapBlockToY(int block);
		
		bool runnerHasAllGold(); // Returns true if runner has picked up all the gold bags
		
		void showAdvanceLevelMsg(XInfo &xinfo); // Display message that we are advancing to next level
		
		void repaintBlock(XInfo &xinfo, int block);
	private:
	
		// Runner's initial location
		int initialXRunner, initialYRunner;
		// Set Goon Initial locations
		vector<int> xCoordsGoons;
		vector<int> yCoordsGoons;
		
		vector<string> levelMap; // Vector of strings that stores information regarding the different components of level
		Runner *runner;
		vector<Goon*> goons;
		
		XInfo *myXInfo;
		
		int runnerLives; // number of lives of runner left
		
		Pixmap goldImage; // Image of gold bag
		bool firstGoldPaint;
		
		int goonGoldDropCount; // counter used to determine when goon should drop gold
		int totalGoldCount; // total number of gold bags in level
		
		int movePixel; // Number of pixels to move for each person
		
		void createRunner(int x, int y, XInfo &xinfo);
		void createGoons(vector<int> xCoords, vector<int> yCoords, XInfo &xinfo);
		
		void drawRunner(XInfo &xinfo);
		void drawGoons(XInfo &xinfo);
		void dropGold(XInfo &xinfo, Goon* goon);
		// Draw Components
		void drawFloor(XInfo &xinfo, int block);
		void drawBar(XInfo &xinfo, int block);
		void drawLadder(XInfo &xinfo, int block);
		void drawGold(XInfo &xinfo, int block);
		
		
		
		int countGoldInLevel(); // Counts the number of gold bags in level
		// Based on starting and ending location, help goon make a decision
		Action makeDecision(Location locStart, Location locEnd, Person* person);
		
		
		char getComponent(int block);
		void setComponent(int block, char blockType);
		void drawComponent(XInfo &xinfo, char comp, int block);
	
		
		
		void displayStaticMessages(XInfo &xinfo); // Display static messages on screen - no repaint required
		
		void readGoldBitmap(XInfo &xinfo); // Reads in the gold image
};

#endif