// Gaurav Jain
// SE 382 - Assignment 1
// Person.cpp

#include <iostream> 

#include "Person.h"
#include "Goon.h"

#define blockWidth 40
#define blockHeight 30

// Constructor
Person::Person(int x, int y, Level *lev, XInfo &xinfo) {
	posX = x;
	posY = y;
	level = lev;
	freeFall = false;
	gold=0;
	moveAction = LEFT;
	leftMovePos =0;
	rightMovePos = 0;
	upMovePos = 0;
	downMovePos = 0;
	goonMoveCount = 1;	 // Every goon is repainted each time a goon moves. When the number of goons is 3, this 
						 // poses a problem as the goons will end up not changing images. This variables solves that
						 
	runnerMoveCount = 1; // When pixels are incremented by 3, the runner image ends up not changing. Therefore, 
			             // we use this special variable to help us solve that boundary condition
			             
	runnerUpCount = 1;	// When pixels are incremented by 2, the runner image ends up not changing when the runner 	is going up. Therefore, 
			             // we use this special variable to help us solve that boundary condition
	
	Person::xinfo = &xinfo;
}

// Destructor - since we don't actually ever create a Person object, we don't need anything in the destructor
Person::~Person() { }

movePoints Person::move(Action moveAction) {
	movePoints movePnts;
	bool legalMove = verifyMove(moveAction); // Determine if the given move is legal or not in the location
	
	this->moveAction = moveAction;
	if(legalMove) {	
		switch(moveAction) {
			case UP:
				// Set extreme coordinates of move - used for repainting
				movePnts.start.leftX = posX;
				movePnts.start.leftY = posY;
				movePnts.start.rightX = posX + 39;
				movePnts.start.rightY = posY + 29;
				movePnts.end.leftX = movePnts.start.leftX;
				movePnts.end.leftY = movePnts.start.leftY - 1;
				movePnts.end.rightX = movePnts.start.rightX;
				movePnts.end.rightY = movePnts.start.rightY - 1;
				
				// Move person up
				posY-=1;
				break;
			case DOWN:
				// Set extreme coordinates of move - used for repainting
				movePnts.start.leftX = posX;
				movePnts.start.leftY = posY;
				movePnts.start.rightX = posX + 39;
				movePnts.start.rightY = posY + 29;
				movePnts.end.leftX = movePnts.start.leftX;
				movePnts.end.leftY = movePnts.start.leftY + 1;
				movePnts.end.rightX = movePnts.start.rightX;
				movePnts.end.rightY = movePnts.start.rightY + 1;
				
				// Move person down
				posY+=1;
				break;
			case LEFT:
				// Set extreme coordinates of move - used for repainting
				movePnts.start.leftX = posX;
				movePnts.start.leftY = posY;
				movePnts.start.rightX = posX + 39;
				movePnts.start.rightY = posY + 29;
				movePnts.end.leftX = movePnts.start.leftX - 1;
				movePnts.end.leftY = movePnts.start.leftY;
				movePnts.end.rightX = movePnts.start.rightX - 1;
				movePnts.end.rightY = movePnts.start.rightY;
				
				// Move person left
				posX-=1;
				// If runner is walking into free space, then start free fall
				if(((level->getBlockType(rightX - 1, rightY) == 'S') || (level->getBlockType(rightX -1, rightY) == 'G')) && ((level->getBlockType(rightX - 1, rightY+1) == 'S') || (level->getBlockType(rightX - 1, rightY + 1) == 'G')) ) {
					posY+=1;
					posX+=1;
					freeFall = true;
				}
				// If runner is walking into free space, such that there is a bar under him/her, then keep falling
				// until you hit the bar
				else if(((level->getBlockType(rightX-1, leftY) == 'S') || (level->getBlockType(rightX-1, leftY) == 'S')) && (level->getBlockType(rightX-1, rightY+1) == 'B')) {
					posY+=1;
					posX+=1;
					freeFall = true;
				}
				// Pickup Gold
				if(level->getBlockType(posX, posY) == 'G') {
					gold++;
					level->setBlockType(posX, posY, 'S');
				}
				break;
			case RIGHT:
				// Set extreme coordinates of move - used for repainting
				movePnts.start.leftX = posX;
				movePnts.start.leftY = posY;
				movePnts.start.rightX = posX + 39;
				movePnts.start.rightY = posY + 29;
				movePnts.end.leftX = movePnts.start.leftX + 1;
				movePnts.end.leftY = movePnts.start.leftY;
				movePnts.end.rightX = movePnts.start.rightX + 1;
				movePnts.end.rightY = movePnts.start.rightY;
				// Move person right
				posX+=1;
				// If walking into free space, then start free fall
				if(((level->getBlockType(leftX + 1, rightY) == 'S') || (level->getBlockType(leftX + 1, rightY) == 'G')) && ((level->getBlockType(leftX + 1, rightY+1) == 'S') || (level->getBlockType(leftX + 1, rightY+1) == 'G'))) {
					posY+=1;
					posX-=1;
					freeFall = true;
				}
				// If runner is walking into free space, such that there is a bar under him/her, then keep falling
				// until you hit the bar
				else if(((level->getBlockType(leftX + 1, leftY) == 'S') || (level->getBlockType(leftX + 1, leftY) == 'S')) && (level->getBlockType(leftX + 1, rightY+1) == 'B')) {
					posY+=1;
					posX+=1;
					freeFall = true;
				}
				
				// Pickup Gold
				if(level->getBlockType(posX, posY) == 'G') {
					gold++;
					level->setBlockType(posX, posY, 'S');
				}
				break;
		}
	}
	else {
		// The person did not move. Set the extreme coordinates accordingly
		movePnts.start.leftX = -1; // Signify that we haven't moved. Don't repaint
		movePnts.start.leftY = posY;
		movePnts.start.rightX = posX + 39;
		movePnts.start.rightY = posY + 29;
		movePnts.end.leftX = movePnts.start.leftX;
		movePnts.end.leftY = movePnts.start.leftY;
		movePnts.end.rightX = movePnts.start.rightX;
		movePnts.end.rightY = movePnts.start.rightY;
	}
	return movePnts;
}

// Return if person is in free fall
bool Person::isFreeFalling() {
	return freeFall;
}

bool Person::verifyMove(Action moveAction) {
	int myUpperLeftBlock = level->mapPointsToBlock(leftX, leftY);
	int myLowerLeftBlock = level->mapPointsToBlock(leftX, rightY);
	int myUpperRightBlock = level->mapPointsToBlock(rightX, leftY);
	int myLowerRightBlock = level->mapPointsToBlock(rightX, rightY);
	
	switch(moveAction) {
		case UP:
			// If a person is above me, then I can't climb
			if(!level->noPerson(level->mapBlockToX(myUpperLeftBlock - (800/blockWidth)), level->mapBlockToY(myUpperLeftBlock - (800/blockWidth))) && (dynamic_cast<Goon*>(this) != 0)) {
			//if(!level->noPerson(leftX, leftY - 1)) {
				return false;
			}
			// As long as both feet are on the ladder, I can climb up
			if((level->getBlockType(myLowerLeftBlock) == 'L') && (level->getBlockType(myLowerRightBlock) == 'L')) {
			//if((level->getBlockType(leftX, rightY) == 'L') && (level->getBlockType(rightX, rightY) == 'L')) {
				if((level->getBlockType(myLowerLeftBlock - (800/blockWidth)) == 'F') || (level->getBlockType(myLowerRightBlock - (800/blockWidth)) == 'F')) {
				return false;
				}
				else {
					return true;
				}
			}
			break;
		case DOWN:
			// If a person is below me, then I can't climb
			if(!level->noPerson(level->mapBlockToX(myLowerLeftBlock + (800/blockWidth)), level->mapBlockToY(myLowerLeftBlock + (800/blockWidth))) && (dynamic_cast<Goon*>(this) != 0)) {
			//if(!level->noPerson(leftX, rightY + 1)) {
				return false;
			}
			// If I am free falling, and I hit a bar, then stop free falling and go hand over hand
			if((level->getBlockType(leftX, leftY) == 'B') && (level->getBlockType(rightX, rightY) == 'B') && (freeFall)) {
				freeFall = false;
				return false;
			}
			// If I am free falling, my feet have hit the bar, but haven't been able to grab the bar with my hands, 
			// then keep free falling
			else if(((level->getBlockType(leftX, leftY) == 'S') || (level->getBlockType(leftX, leftY) == 'G')) && (level->getBlockType(rightX, rightY) == 'B') && (freeFall)) {
				return true;
			}
			// If there is no floor below me, then keep going down
			else if((level->getBlockType(rightX, rightY+1) != 'F') && (level->getBlockType(leftX, rightY+1) != 'F')) {
				return true;
			}
			else {
				freeFall = false; // couldn't be free falling
				return false;
			}
			break;
		case LEFT:
			// If there is a person left of me, then I can't go left
			if(!level->noPerson(level->mapBlockToX(myUpperLeftBlock - 1), level->mapBlockToY(myUpperLeftBlock - 1))) {
				return false;
			}
			// If there is no floor left of me, then go left
			if(((level->getBlockType(leftX-1, leftY) != 'F') && (level->getBlockType(leftX-1, rightY) != 'F')) && (leftX>0)) {
				return true;
			}
			break;
		case RIGHT:
			// If there is a person right of me, then I can't go right
			if(!level->noPerson(level->mapBlockToX(myUpperRightBlock + 1), level->mapBlockToY(myUpperRightBlock +1))) {
			//if(!level->noPerson(rightX + 1, rightY)) {
				return false;
			}
			// If there is no floor right of me, then I can go right
			if(((level->getBlockType(rightX+1, rightY) != 'F') && (level->getBlockType(rightX+1, leftY) != 'F')) && (rightX<800)) {
				return true;
			}
			break;
	}
	return false;
}

// Get location of a person - top left corner coordinates only
Location Person::getLoc() {
	Location myLoc;
	myLoc.xCoord = posX;
	myLoc.yCoord = posY;	
	return myLoc;
}

// Get four extremities of a person
Extremeties Person::getExts() {
	Extremeties exts;
	exts.leftX = leftX;
	exts.leftY = leftY;
	exts.rightX = rightX;
	exts.rightY = rightY;
	return exts;
}

// Return number of gold bags owned
int Person::getNumGold() {
	return gold;
}

// Return direction the person is running in
Action Person::getDirection() {
	return moveAction;
}

// Customized paint methods - used to load different bitmaps
void Person::paintLeft(XInfo &xinfo) {
	// The goons have a cycle of three repaints for every goon. Thus this check below is needed to be able
	// to distinguish between each position while they are running 
	bool isGoon = (dynamic_cast<Goon*>(this) != 0);
	if(isGoon) {
		if(goonMoveCount%3 == 0) {
			leftMovePos++;
			leftMovePos%=3;
		}	
		goonMoveCount++;
	}
	else {
			leftMovePos++;
			leftMovePos%=9;
	}
	
	if((leftMovePos >= 3) && (leftMovePos < 6)) {
		XSetClipMask(xinfo.display, xinfo.gc, leftPos2Mask);
		XSetClipOrigin(xinfo.display, xinfo.gc, posX, posY);
		XCopyPlane(xinfo.display, leftPos2, xinfo.window, xinfo.gc, 0, 0, 35, 30, posX, posY, 1);
		XSetClipMask(xinfo.display, xinfo.gc, None); // Set Clipping mask to none so you can paint outisde the mask
	}
	else if((leftMovePos >= 6) && (leftMovePos < 9)) {
		XSetClipMask(xinfo.display, xinfo.gc, leftPos3Mask);
		XSetClipOrigin(xinfo.display, xinfo.gc, posX, posY);
		XCopyPlane(xinfo.display, leftPos3, xinfo.window, xinfo.gc, 0, 0, 35, 30, posX, posY, 1);
		XSetClipMask(xinfo.display, xinfo.gc, None); // Set Clipping mask to none so you can paint outisde the mask
	}
	else {
		XSetClipMask(xinfo.display, xinfo.gc, leftPos1Mask);
		XSetClipOrigin(xinfo.display, xinfo.gc, posX, posY);
		XCopyPlane(xinfo.display, leftPos1, xinfo.window, xinfo.gc, 0, 0, 35, 30, posX, posY, 1);
		XSetClipMask(xinfo.display, xinfo.gc, None); // Set Clipping mask to none so you can paint outisde the mask
	}
}

// Load bitmap for moving right
void Person::paintRight(XInfo &xinfo) {
	// The goons have a cycle of three repaints for every goon. Thus this check below is needed to be able
	// to distinguish between each position while they are running 
	bool isGoon = (dynamic_cast<Goon*>(this) != 0);
	if(isGoon) {
		if(goonMoveCount%3 == 0) {
			rightMovePos++;
			rightMovePos%=3;
		}	
		goonMoveCount++;
	}
	else {
		rightMovePos++;
		rightMovePos%=9;
	}
	
	if((rightMovePos >= 3) && (rightMovePos < 6)) {
		XSetClipMask(xinfo.display, xinfo.gc, rightPos2Mask);
		XSetClipOrigin(xinfo.display, xinfo.gc, posX, posY);
		XCopyPlane(xinfo.display, rightPos2, xinfo.window, xinfo.gc, 0, 0, 35, 30, posX, posY, 1);
		XSetClipMask(xinfo.display, xinfo.gc, None); // Set Clipping mask to none so you can paint outisde the mask
	}
	else if((rightMovePos >=6) && (rightMovePos < 9)) {
		XSetClipMask(xinfo.display, xinfo.gc, rightPos3Mask);
		XSetClipOrigin(xinfo.display, xinfo.gc, posX, posY);
		XCopyPlane(xinfo.display, rightPos3, xinfo.window, xinfo.gc, 0, 0, 35, 30, posX, posY, 1);
		XSetClipMask(xinfo.display, xinfo.gc, None); // Set Clipping mask to none so you can paint outisde the mask
	}
	else {
		XSetClipMask(xinfo.display, xinfo.gc, rightPos1Mask);
		XSetClipOrigin(xinfo.display, xinfo.gc, posX, posY);
		XCopyPlane(xinfo.display, rightPos1, xinfo.window, xinfo.gc, 0, 0, 35, 30, posX, posY, 1);
		XSetClipMask(xinfo.display, xinfo.gc, None); // Set Clipping mask to none so you can paint outisde the mask
	}
}
		
		
void Person::paintUp(XInfo &xinfo) {
	// The goons have a cycle of three repaints for every goon. Thus this check below is needed to be able
	// to distinguish between each position while they are running 
	bool isGoon = (dynamic_cast<Goon*>(this) != 0);
	if(isGoon) {
		if(goonMoveCount % 2 == 0) {
			upMovePos++;
			upMovePos%=40;
		}
		goonMoveCount++;
	}
	else {
		upMovePos++;
		upMovePos%=40;
	}
	
	if(upMovePos < 20) {
		XSetClipMask(xinfo.display, xinfo.gc, upPos2Mask);
		XSetClipOrigin(xinfo.display, xinfo.gc, posX, posY);
		XCopyPlane(xinfo.display, upPos2, xinfo.window, xinfo.gc, 0, 0, 35, 30, posX, posY, 1);
		XSetClipMask(xinfo.display, xinfo.gc, None); // Set Clipping mask to none so you can paint outisde the mask
	}
	else {
		XSetClipMask(xinfo.display, xinfo.gc, upPos1Mask);
		XSetClipOrigin(xinfo.display, xinfo.gc, posX, posY);
		XCopyPlane(xinfo.display, upPos1, xinfo.window, xinfo.gc, 0, 0, 35, 30, posX, posY, 1);
		XSetClipMask(xinfo.display, xinfo.gc, None); // Set Clipping mask to none so you can paint outisde the mask
	}
}


void Person::paintDown(XInfo &xinfo) {
	// The goons have a cycle of three repaints for every goon. Thus this check below is needed to be able
	// to distinguish between each position while they are running 
	bool isGoon = (dynamic_cast<Goon*>(this) != 0);
	if(isGoon) {
		if(goonMoveCount % 3 == 0) {
			downMovePos++;
		}
		goonMoveCount++;
	}
	else {
		downMovePos++;
		downMovePos%=40;
	}
	
	if(downMovePos < 20) {
		XSetClipMask(xinfo.display, xinfo.gc, downPos1Mask);
		XSetClipOrigin(xinfo.display, xinfo.gc, posX, posY);
		XCopyPlane(xinfo.display, downPos1, xinfo.window, xinfo.gc, 0, 0, 35, 30, posX, posY, 1);
		XSetClipMask(xinfo.display, xinfo.gc, None); // Set Clipping mask to none so you can paint outisde the mask
	}
	else {
		XSetClipMask(xinfo.display, xinfo.gc, downPos2Mask);
		XSetClipOrigin(xinfo.display, xinfo.gc, posX, posY);
		XCopyPlane(xinfo.display, downPos2, xinfo.window, xinfo.gc, 0, 0, 35, 30, posX, posY, 1);
		XSetClipMask(xinfo.display, xinfo.gc, None); // Set Clipping mask to none so you can paint outisde the mask
	}
}

// Load up image for bar
void Person::paintBar(XInfo &xinfo) {
	XSetClipMask(xinfo.display, xinfo.gc, bar1Mask);
	XSetClipOrigin(xinfo.display, xinfo.gc, posX, posY);
	XCopyPlane(xinfo.display, bar1, xinfo.window, xinfo.gc, 0, 0, 35, 30, posX, posY, 1);
	XSetClipMask(xinfo.display, xinfo.gc, None); // Set Clipping mask to none so you can paint outisde the mask
}

