// Gaurav Jain
// SE 382 - Assignment 1
// Person.h

// Superclass of Runner and Goon. Implements common functions such as paint and stores a few variables in protected
// mode

#ifndef _PERSON_H_
#define _PERSON_H_

#include "Displayable.h"
#include "global_enum_defs.h"
#include "Level.h"

using namespace std;

class Level;

struct movePoints {
	Extremeties start;
	Extremeties end;
};

class Person : public Displayable {
	public:
		Person(int x, int y, Level *lev, XInfo &xinfo);
		~Person();
		virtual void paint(XInfo &xinfo) = 0; /* abstract function - needs to be implemented by every non-abstract class */
		Location getLoc(); // Get location of a person
		Extremeties getExts(); // Get 4 extremeties of a person
		movePoints move(Action moveAction);
		bool isFreeFalling();
		bool verifyMove(Action moveAction);	
		int getNumGold(); // return number of gold bags owned
		Action getDirection(); // returns the direction the person is running in
		
			
	private:
		int goonMoveCount;   // Function of this variable described in constructor of Person
		int runnerMoveCount; // Function of this variable described in constructor of Person
		int runnerUpCount; // Function of this variable described in constructor of Person
		
	protected:
		int posX, posY;
		Level *level;
		bool freeFall;
		int leftX, leftY, rightX, rightY; // Four extreme dimensions of the player
		int gold; // number of gold boxes owned
		Action moveAction;
		int leftMovePos, rightMovePos, upMovePos, downMovePos; // keeps track of what bitmap to load when running left, right, up or down
		
		// Pix maps for reading bitmap files
		Pixmap leftPos1;
		Pixmap leftPos2;
		Pixmap leftPos3;
		
		Pixmap leftPos1Mask;
		Pixmap leftPos2Mask;
		Pixmap leftPos3Mask;
		
		Pixmap rightPos1;
		Pixmap rightPos2;
		Pixmap rightPos3;
		
		Pixmap rightPos1Mask;
		Pixmap rightPos2Mask;
		Pixmap rightPos3Mask;
		
		Pixmap upPos1;
		Pixmap upPos2;
		
		Pixmap upPos1Mask;
		Pixmap upPos2Mask;
		
		Pixmap downPos1;
		Pixmap downPos2;
		
		Pixmap downPos1Mask;
		Pixmap downPos2Mask;
		
		Pixmap bar1;
		
		Pixmap bar1Mask;
		
		// Customized paint functions
		void paintRight(XInfo &xinfo);
		void paintLeft(XInfo &xinfo);
		void paintUp(XInfo &xinfo);
		void paintDown(XInfo &xinfo);
		void paintBar(XInfo &xinfo);
		
		// xinfo reference needed for destructor
		XInfo *xinfo;
};

#endif