// Gaurav Jain
// SE 382 - Assignment 1
// Runner.cpp

#include "Runner.h"

#include <iostream>

// Constructor
Runner::Runner(int x, int y, Level *lev, XInfo &xinfo) : Person(x, y, lev, xinfo) {
	freeFall = false;
	firstPaint = true;
	lives = 5;
}

// Destructor
Runner::~Runner() {
	
	// Free all the pixmaps
	XFreePixmap(xinfo->display, leftPos1);
	XFreePixmap(xinfo->display, leftPos2);
	XFreePixmap(xinfo->display, leftPos3);
	XFreePixmap(xinfo->display, rightPos1);
	XFreePixmap(xinfo->display, rightPos2);
	XFreePixmap(xinfo->display, rightPos3);
	XFreePixmap(xinfo->display, upPos1);
	XFreePixmap(xinfo->display, upPos2);
	XFreePixmap(xinfo->display, downPos1);
	XFreePixmap(xinfo->display, downPos2);
	XFreePixmap(xinfo->display, bar1);
}
	
// Paint the runner. Based on direction/location of runner call a different paint method that will in turn
// call a different bitmap file to load
void Runner::paint(XInfo &xinfo) {
	if(firstPaint) {
		readBitMaps(xinfo);
		firstPaint = false;
		paint(xinfo);
	} 
	else {
		if(level->getBlockType(getLoc().xCoord, getLoc().yCoord) == 'B') {
			paintBar(xinfo);
		}
		else if(moveAction == LEFT) {
			paintLeft(xinfo);
		}
		else if(moveAction == RIGHT) {
			paintRight(xinfo);
		}
		else if(moveAction == UP) {
			paintUp(xinfo);
		}
		else if(moveAction == DOWN) {
			paintDown(xinfo);
		}
		leftX=posX; leftY=posY; rightX=posX+29; rightY=posY+29;
	}
}
		
void Runner::readBitMaps(XInfo &xinfo) {
	// Read all Image files that I need to animate runner
	unsigned int pic_width = 0, pic_height = 0;
	int x_hot_return, y_hot_return;
	
	// Read bitmaps for moving left
	XReadBitmapFile(xinfo.display, xinfo.window, "./images/left_pos1.xbm", &pic_width, &pic_height, &leftPos1, &x_hot_return, &y_hot_return);
	XReadBitmapFile(xinfo.display, xinfo.window, "./images/left_pos2.xbm", &pic_width, &pic_height, &leftPos2, &x_hot_return, &y_hot_return);
	XReadBitmapFile(xinfo.display, xinfo.window, "./images/left_pos3.xbm", &pic_width, &pic_height, &leftPos3, &x_hot_return, &y_hot_return);
	
	// Read bitmap masks for moving left
	XReadBitmapFile(xinfo.display, xinfo.window, "./images/left_pos1_mask.xbm", &pic_width, &pic_height, &leftPos1Mask, &x_hot_return, &y_hot_return);
	XReadBitmapFile(xinfo.display, xinfo.window, "./images/left_pos2_mask.xbm", &pic_width, &pic_height, &leftPos2Mask, &x_hot_return, &y_hot_return);
	XReadBitmapFile(xinfo.display, xinfo.window, "./images/left_pos3_mask.xbm", &pic_width, &pic_height, &leftPos3Mask, &x_hot_return, &y_hot_return);
	
	// Read bitmaps for moving right
	XReadBitmapFile(xinfo.display, xinfo.window, "./images/right_pos1.xbm", &pic_width, &pic_height, &rightPos1, &x_hot_return, &y_hot_return);
	XReadBitmapFile(xinfo.display, xinfo.window, "./images/right_pos2.xbm", &pic_width, &pic_height, &rightPos2, &x_hot_return, &y_hot_return);
	XReadBitmapFile(xinfo.display, xinfo.window, "./images/right_pos3.xbm", &pic_width, &pic_height, &rightPos3, &x_hot_return, &y_hot_return);
	
	// Read bitmaps for moving right
	XReadBitmapFile(xinfo.display, xinfo.window, "./images/right_pos1_mask.xbm", &pic_width, &pic_height, &rightPos1Mask, &x_hot_return, &y_hot_return);
	XReadBitmapFile(xinfo.display, xinfo.window, "./images/right_pos2_mask.xbm", &pic_width, &pic_height, &rightPos2Mask, &x_hot_return, &y_hot_return);
	XReadBitmapFile(xinfo.display, xinfo.window, "./images/right_pos3_mask.xbm", &pic_width, &pic_height, &rightPos3Mask, &x_hot_return, &y_hot_return);
	
	// Read bitmaps for going up
	XReadBitmapFile(xinfo.display, xinfo.window, "./images/up_pos1.xbm", &pic_width, &pic_height, &upPos1, &x_hot_return, &y_hot_return);
	XReadBitmapFile(xinfo.display, xinfo.window, "./images/up_pos2.xbm", &pic_width, &pic_height, &upPos2, &x_hot_return, &y_hot_return);
	
	// Read bitmaps for going up
	XReadBitmapFile(xinfo.display, xinfo.window, "./images/up_pos1_mask.xbm", &pic_width, &pic_height, &upPos1Mask, &x_hot_return, &y_hot_return);
	XReadBitmapFile(xinfo.display, xinfo.window, "./images/up_pos2_mask.xbm", &pic_width, &pic_height, &upPos2Mask, &x_hot_return, &y_hot_return);
		
	// Read bitmaps for going down
	XReadBitmapFile(xinfo.display, xinfo.window, "./images/up_pos2.xbm", &pic_width, &pic_height, &downPos1, &x_hot_return, &y_hot_return);
	XReadBitmapFile(xinfo.display, xinfo.window, "./images/up_pos1.xbm", &pic_width, &pic_height, &downPos2, &x_hot_return, &y_hot_return);
	
	// Read bitmaps for going down
	XReadBitmapFile(xinfo.display, xinfo.window, "./images/up_pos2_mask.xbm", &pic_width, &pic_height, &downPos1Mask, &x_hot_return, &y_hot_return);
	XReadBitmapFile(xinfo.display, xinfo.window, "./images/up_pos1_mask.xbm", &pic_width, &pic_height, &downPos2Mask, &x_hot_return, &y_hot_return);
	
	// Read bitmap for when on bar
	XReadBitmapFile(xinfo.display, xinfo.window, "./images/runner_bar.xbm", &pic_width, &pic_height, &bar1, &x_hot_return, &y_hot_return);
	
	// Read bitmap for when on bar
	XReadBitmapFile(xinfo.display, xinfo.window, "./images/runner_bar_mask.xbm", &pic_width, &pic_height, &bar1Mask, &x_hot_return, &y_hot_return);

}	