// Gaurav Jain
// SE 382 - Assignment 1
// main.cpp

// This is the main class that contains the event loop

#include <iostream>
#include <string>
#include <fstream>
#include <list>
#include <sys/time.h>

#include "Level.h"
#include "Displayable.h"
#include "global_enum_defs.h"


/*
 * Header files for X functions
 */
#include <X11/Xlib.h>
#include <X11/Xutil.h>


// Constant frames per second
#define FPS 30

using namespace std;

/*
 * Defines for window geometry and keyboard events
 */
 
const int Border = 5;
const int BufferSize = 10;

int movePixel = 1; // number of pixels to move in each move
int numLevel = 0; // keeps track of what level we are on currently

vector<string> levelsVector; // vector that stores the name of files for each level
bool firstLevel = true; /* flag variable that indicates the program to read the Levels.txt file to initialize level vector */
bool runnerHasAllGold = false; // flag variable indicating if runner has picked up all the gold yet or not
bool advanceToNextLevel = false; // flag indicating if player wants to advance to next level forcibly
/*
 * Function to put out a message on error exits.
 */
void error( string str ) {
  cerr << str << endl;
  exit(0);
}

/* -=-=-=-=- This function reads the Levels.txt file and creates a vector of names of each level file -=-=-- */

void initializeLevelsVector() {
	ifstream infile;
	string fileName = "Levels.txt";
	string levelName;
	infile.open(fileName.c_str());
	
	while(true) {
		infile >> levelName;
		if(infile.fail()) {
			infile.close();
			break;
		}
		levelsVector.push_back(levelName);
	}
}

/* -=-=-=-=- End: initializeLevelsVector() -=--=-=-=-=- */

/* -=--=-=- Returns the name of file of next level -=-=-=-=-=- */

string getNextLevel() {
	string retString;
	if(firstLevel) {
		initializeLevelsVector();
		firstLevel = false;
	}
	
	if(numLevel < levelsVector.size()) {
		retString = levelsVector[numLevel];
		numLevel++;
		return retString;
	}
	else {
		return "";
	}
}

/* -=-=-=-=-= End: getNextLevel() -=-=-=--=-=-=-=-=-=-=- */

/* -=-=- This method dynamically updates the level
   Floor becomes a ladder, a ladder becomes a bar and a bar becomes a floor */
   
void toggleComponent(XInfo &xinfo, int mouseX, int mouseY, Level* curLevel) {
	
	if(curLevel->noPersonAtLoc(mouseX, mouseY)) {
		if(curLevel->getBlockType(mouseX, mouseY) == 'S') {
			curLevel->setBlockType(mouseX, mouseY, 'F');
		}
		else if(curLevel->getBlockType(mouseX, mouseY) == 'F') {
			curLevel->setBlockType(mouseX, mouseY, 'L');
		}
		else if(curLevel->getBlockType(mouseX, mouseY) == 'L') {
			curLevel->setBlockType(mouseX, mouseY, 'B');
		}
		else if(curLevel->getBlockType(mouseX, mouseY) == 'B') {
			curLevel->setBlockType(mouseX, mouseY, 'S');
		}
		
		curLevel->repaintBlock(xinfo, curLevel->mapPointsToBlock(mouseX, mouseY));
	}
}

/* -=-=-- End: toggleComponent() -==--=-=-=-=- */

/*
 * The loop responding to events from the user. 
 */
void eventloop(XInfo &xinfo)
{  
   // Local variables needed by event loop
   XEvent event;
   KeySym key;
   char text[BufferSize];
   Action moveAction = LEFT;
   bool moveActionCond = false; // boolean that tells us if the move direction has been changed by key press
   bool collision = false; // flag variable that indicates if a collission has occurred
   int lives = 5; // number of runner lives
   bool runnerHasAllGold = false; // flag indicating if runner has picked up all gold bags or not
   
   int mouseX = 0;
   int mouseY = 0; // mouse event coordinates stored in here
  
   string curLevelString = getNextLevel();
   // Create new level
   Level *curLevel = new Level(curLevelString, xinfo, lives);
   while( true ) 
   {  
	// Event loop
   if((XEventsQueued(xinfo.display, QueuedAfterFlush)) > 0) {
		  XNextEvent( xinfo.display, &event );
		  //moveActionCond = false;
		  switch( event.type ) {
		  /*
		   * Repaint the window on expose events.
		   */
		  case Expose:
				 if ( event.xexpose.count == 0 ) 
				 {  curLevel->paint(xinfo);
				 }
				 break;
			 
		  /*
		   * Add an item where the mouse is clicked, and repaint.
		   */
		  case ButtonPress:
				mouseX = event.xbutton.x; // We take the coordinates of the point where the mouse is clicked
				mouseY = event.xbutton.y; // and try to take the runner as close to that as possible
				toggleComponent(xinfo, mouseX, mouseY, curLevel);					
				break;
		  /*
		   * Exit when 'q' is typed.
		   * Arguments for XLookupString :
		   *                 event - the keyboard event
		   *                 text - buffer into which text will be written
		   *                 BufferSize - size of text buffer
		   *                 key - workstation independent key symbol
		   *                 0 - pointer to a composeStatus structure
		   */
		  case KeyPress:
			 int i = XLookupString( (XKeyEvent *)&event, text, BufferSize, &key, 0 );
			 if ( i == 1 ) {
				 if( text[0] == 'q' )
				 {  
					error( "Terminated normally." );
				 }
				 
				 else if( text[0] == 'f' ) {    // Increase speed
				    // Go to a max of 5 pixels
					if(movePixel < 4)
						movePixel++;
					curLevel->setMovePixel(movePixel); // Tells the level the number of pixel each person should move
					
				 }
				 
				 else if( text[0] == 's' ) {  // Decrease speed
					// Go to a min of 1 pixel
					if(movePixel > 1) 
						movePixel--;
					curLevel->setMovePixel(movePixel); // Tells the level the number of pixel each person should move
				 }
				 
				 else if( text[0] == 'p' ) {  // Pause/Unpause
					moveActionCond = !moveActionCond;
				 }
				 
				 else if( text[0] == 'j' ) // Go left
				 { 
					// Move the runner left
					moveAction = LEFT;
					moveActionCond = true;
				 }
				 else if( text[0] == 'l' )  // Go right
				 {
					// Move the runner right
					moveAction = RIGHT;
					moveActionCond = true;
				 }
				 else if( text[0] == 'i' )  // Go up
				 {
					// Move the runner up
					moveAction = UP;
					moveActionCond = true;
				 }
				 else if( text[0] == 'k' ) // Go down
				 {
					// Move the runner down
					moveAction = DOWN;
					moveActionCond = true;
				 }
				 
				 else if( text[0] == 'g' ) // Kill random goon - make game easier
				 {
					// Get rid of random goon
					curLevel->getRidOfRandomGoon(xinfo);
				}
				
				else if( text[0] == 'c' ) // Increase life by 1
				{
					// Increase runner life by 1 - stretch game if you are having a lot of fun
					lives++;
					curLevel->setRunnerLives(xinfo, lives);
				}
				else if( text[0] == 'a' ) // Advance to next level
				{
					advanceToNextLevel = true;
				}
			}
			break;
		  }
      }
      
      // Flush
      XFlush(xinfo.display);
      
      struct timeval timeStart, timeEnd;
      gettimeofday(&timeStart, NULL); // Figure out current time
      
      if(moveActionCond)
      {
		
		for(int i=1; i<=movePixel; i++) {  // Move multiple pixels if required
			curLevel->moveRunner(xinfo, moveAction); // Move the runner
			runnerHasAllGold = curLevel->runnerHasAllGold();
			if(runnerHasAllGold)
				break;
			curLevel->moveGoons(xinfo);
			collision = curLevel->checkCollision();
			if(collision)
				break;
		}
		if(collision) { // If collision has occured then start new level
			delete curLevel;
			lives--;
			if(lives>0) {
				curLevel = new Level(curLevelString, xinfo, lives);
			}
			else {
				error ("Game Over. \nAll runner lives used. \nThanks for playing. ");
			}
			collision = false;
			moveActionCond = false;
		}
					
	  }
      
      if(runnerHasAllGold || advanceToNextLevel) { // If runnner has all gold then advance to next level
			advanceToNextLevel = false; // reinitialize flag variables
			runnerHasAllGold = false; 
			curLevelString = getNextLevel(); // find out what the name of the file is for next level
			if(curLevelString != "")
				curLevel->showAdvanceLevelMsg(xinfo); // displays message that we are advancing to next level
			delete curLevel; // Delete current level
			if(curLevelString != "") {
				curLevel = new Level(curLevelString, xinfo, lives); // advance to next level
				moveActionCond = false;
			}
			else {
				error ("You have successfully completed the game.\nCongratulations!");
			}
	  }
		
      gettimeofday(&timeEnd, NULL); // Get end time and figure out how much we want to sleep for
      int temp = (int)(((1000000/FPS) - (timeEnd.tv_usec - timeStart.tv_usec)));
      if(temp > 0)
		usleep(temp);
   }
   
}

/*
 * Create the window;  initialize X.
 */
void initX(int argc, char *argv[], XInfo &xinfo)
{  XSizeHints hints;
   unsigned long background, foreground;
   int screen;

   /*
    * Display opening uses the DISPLAY  environment variable.
    * It can go wrong if DISPLAY isn't set, or you don't have permission.
    */  
   xinfo.display = XOpenDisplay( "" );
   if ( !xinfo.display )
   { error( "Can't open display." );
   }

   /*
    * Find out some things about the display you're using.
    */
   screen = DefaultScreen( xinfo.display );
   background = WhitePixel( xinfo.display, screen );
   foreground = BlackPixel( xinfo.display, screen );

   /*
    * Set up hints and properties for the window manager, and open a window.
    * Arguments to XCreateSimpleWindow :
    *                 display - the display on which the window is opened
    *                 parent - the parent of the window in the window tree
    *                 x,y - the position of the upper left corner
    *                 width, height - the size of the window
    *                 Border - the width of the window border
    *                 foreground - the colour of the window border
    *                 background - the colour of the window background.
    * Arguments to XSetStandardProperties :
    *                 display - the display on which the window exists
    *                 window - the window whose properties are set
    *                 LodeRunner - the title of the window
    *                 LodeRunner2 - the title of the icon
    *                 none - a pixmap for the icon
    *                 argv, argc - a comand to run in the window
    *                 hints - sizes to use for the window.
    */
   hints.x = 100;
   hints.y = 100;
   hints.width = 800;
   hints.height = 600;
   hints.flags = PPosition | PSize;
   xinfo.window = XCreateSimpleWindow( xinfo.display, DefaultRootWindow( xinfo.display ),
                                hints.x, hints.y, hints.width, hints.height,
                                Border, foreground, background );
   XSetStandardProperties( xinfo.display, xinfo.window, "LodeRunner By Gaurav Jain", "LodeRunner2", None,
                                argv, argc, &hints );

   /*
    * Get a graphics context and set the drawing colours.
    * Arguments to XCreateGC
    *           display - which uses this GC
    *           window - which uses this GC
    *           GCflags - flags which determine which parts of the GC are used
    *           GCdata - a struct that fills in GC data at initialization.
    */
   xinfo.gc = XCreateGC (xinfo.display, xinfo.window, 0, 0 );
   XSetBackground( xinfo.display, xinfo.gc, background );
   XSetForeground( xinfo.display, xinfo.gc, foreground );

   /*
    * Tell the window manager what input you want.
    */
   XSelectInput( xinfo.display, xinfo.window,
                ButtonPressMask | KeyPressMask | ExposureMask );

   /*
    * Put the window on the screen.
    */
   XMapRaised( xinfo.display, xinfo.window );
   /* Disable Graphics Expose Events */
   
   XSetGraphicsExposures(xinfo.display, xinfo.gc, false);
}



/*
 * Start executing here.
 *   First initialize window.
 *   Next loop responding to events.
 *   Exit forcing window manager to clean up - cheesy, but easy.
 */
int main ( int argc, char *argv[] ) 
{  XInfo xinfo;
   initX(argc, argv, xinfo);
   eventloop(xinfo);
}
